;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'dired-markers '("dired"))
(custom-add-loads 'environment '("dired"))
(custom-add-loads 'tools '("diff"))
(custom-add-loads 'dired-behavior '("dired"))
(custom-add-loads 'diff '("diff"))
(custom-add-loads 'dired-hooks '("dired"))
(custom-add-loads 'dired '("dired-faces" "dired-shell" "dired"))
(custom-add-loads 'dired-programs '("dired"))
(custom-add-loads 'dired-faces '("dired-faces"))
(custom-add-loads 'dired-file-names '("dired"))

;;; custom-load.el ends here
